#pragma once







namespace uCalcNameSpace {
    using namespace std;
    using namespace System;
    using namespace System::Runtime::InteropServices;
    using namespace System::Collections::Generic;

#if _WIN64
#define uCalcDLL "uCalc64.dll"
#else
#define uCalcDLL "uCalc32.dll"
#endif

#define MARSHALSTR [In,Out,MarshalAs(UnmanagedType::AnsiBStr)]String ^
#define CONSTSTRING String ^
#define STRING String ^
#define RETURNSTRING(arg) return Marshal::PtrToStringAnsi(arg)
#define DELEGATE uCalcCallback
#define EMPTYSTR gcnew String("")
#define STR_RETURN IntPtr
#define BYREF %
#define UCCALLBACK uCalcDelegate ^
#define ucPinAddr(UserFunction) (uCalc::Addresses->Add(gcnew uCalcDelegate(&UserFunction)), uCalc::Addresses[uCalc::Addresses->Count-1])



    typedef void (*uCalcPtr)(bool, char, int, long, float, double);
    typedef void (__stdcall *uCalcCallback)(uCalcPtr);


    /// <summary>S: Data types that come with uCalc</summary>
    /// <remarks>These are built-in data types that come with uCalc</remarks>
    enum BuiltInTypesEnum : Int32 {
        Type_Unspecified,
        Type_Auto,
        /// <summary> Boolean -- </summary>
        Type_Boolean,
        /// <summary> Byte +++ Rename these to things like Type_Int8</summary>
        Type_Integer_8,
        /// <summary> Byte</summary>
        Type_Integer_8u,
        /// <summary> Int16</summary>
        Type_Integer_16,
        /// <summary> Int16u</summary>
        Type_Integer_16u,
        /// <summary> Int32</summary>
        Type_Integer_32,
        /// <summary> Int32u</summary>
        Type_Integer_32u,
        /// <summary> Int64</summary>
        Type_Integer_64,
        /// <summary> Int64u</summary>
        Type_Integer_64u,
        /// <summary> Int128</summary>
        Type_Integer_128,
        /// <summary> Int128u</summary>
        Type_Integer_128u,
        /// <summary> Single</summary>
        Type_Float_Single,
        /// <summary> Double</summary>
        Type_Float_Double,
        /// <summary> Extended</summary>
        Type_Float_Extended,
        Type_Size_t,
        Type_Complex,
        Type_Decimal,
        Type_BigNum,
        Type_Date,
        Type_String,
        /// <summary> LPCSTR</summary>
        Type_String_LPCSTR,
        /// <summary> FixedStr</summary>
        Type_String_Fixed,
        /// <summary> WideStr</summary>
        Type_String_Wide,
        /// <summary> StackList</summary>
        Type_List_Stack,
        /// <summary> TableList</summary>
        Type_List_Table,
        /// <summary> SortedList</summary>
        Type_List_Sorted,
        Type_Null,
        Type_Void,
        Type_Any,
        Type_Pointer,
        Type_Omni,
        Type_UserDefined,
        Type_DataType,
        Type_Namespace,
        Type_IntPtr,
        Type_ADDR,
        Type_uCalcPtr,
        Type_Item,
        Type_ParsedExpr,
        Type_uCalcCallback,
        Type_Object,
        Type_Intrinsic
    };

    /// <summary>S: Data type constructors</summary>
    enum DataTypeConstruction : Int32 {
        ByteSize,
        AllocArray,
        AllocScalar,
        AllocByPtr,
        ReleaseArray,
        ReleaseScalar,
        ResetScalar,
        ResetArray,
        SetArrayValue,
        SetScalarValue,
        Swap
    };

    /// <summary>S: Error hander responses</summary>
    enum ErrorHandlerResponseEnum : Int32 {
        /// <summary> Immediately stops parsing</summary>
        ErrorAbort,
        /// <summary> Continues parsing (after the error is resolved)</summary>
        ErrorResume,
        /// <summary> Raises the same error again for the next error handler to take care of</summary>
        ErrorReRaise
    };

    /// <summary>S: Error numbers that uCalc may return</summary>
    enum ErrorNumberEnum : Int32 {
        /// <summary> No error</summary>
        Err_None,
        /// <summary> Dynamically defined user error</summary>
        Err_Dynamically_Defined,
        /// <summary> Syntax error</summary>
        Err_Syntax_Error,
        /// <summary> Undefined identifier</summary>
        Err_Undefined_Identifier,
        /// <summary> File not found</summary>
        Err_FileNotFound,
        /// <summary> Unrecognized token</summary>
        Err_Unrecognized_Token,
        /// <summary> Unrecognized command</summary>
        Err_Unrecognized_Command,
        /// <summary> Data type mismatch</summary>
        Err_Datatype_Mismatch,
        /// <summary> Invalid number of arguments</summary>
        Err_Invalid_Argument_Count,
        /// <summary> Invalid definition</summary>
        Err_Invalid_Definition,
        /// <summary> Code block delimiter error</summary>
        Err_CodeBlock_Error,
        /// <summary> Undefined callback</summary>
        Err_Undefined_Callback,
        /// <summary> Error message already defined</summary>
        Err_ErrorMessageAlreadyDefined,
        /// <summary> This item cannot be modified</summary>
        Err_ItemCannotBeModified,
        /// <summary> Unrecognized class member</summary>
        Err_Unrecognized_Class_Member,
        /// <summary> Unrecognized namespace member</summary>
        Err_Unrecognized_Namespace_Member,
        /// <summary> Unbalanced quote</summary>
        Err_Unbalanced_Quote,
        /// <summary> Value cannot be assigned here</summary>
        Err_ValueCannotBeAssigned,
        /// <summary> Array or list bound exceeded</summary>
        Err_Array_Bounds_Exceeded,
        /// <summary> Division by 0</summary>
        Err_Float_Division_By_Zero,
        /// <summary> Inexact result</summary>
        Err_Float_Inexact_Result,
        /// <summary> Invalid operation</summary>
        Err_Float_Invalid_Operation,
        /// <summary> Floating point overflow</summary>
        Err_Float_Overflow,
        /// <summary> Floating point underflow</summary>
        Err_Float_Underflow,
        Err_Customized1 = 100,
        Err_Customized2,
        Err_Customized3,
        Err_Customized4,
        Err_Customized5,
        Err_Customized6,
        Err_Customized7,
        Err_Customized8,
        Err_Customized9,
        Err_Customized10,
        Err_Customized11,
        Err_Customized12,
        Err_Customized13,
        Err_Customized14,
        Err_Customized15,
        Err_Customized16,
        Err_Customized17,
        Err_Customized18,
        Err_Customized19,
        Err_Customized20
    };

    /// <summary>S: Operator grouping associativity</summary>
    enum GroupingEnum : Int32 {
        GroupLeftToRight,
        GroupRightToLeft
    };

    /// <summary>S: Properties that a uCalc item may have</summary>
    /// <remarks>
    ///Items defined with uCalc typically have a combination of properties.  A variable defined
    ///with DefineVariable for instance will have the ItemIsEnum.Variable property set.  An operator
    ///will have the ItemIsEnum.Operator property set.  If it's an infix operator, in addition to
    ///having the ItemIsEnum.Operator property set, ItemIsEnum.Infix will also be set.
    ///property will also be set.
    ///
    ///The IsProperty function can determine if a given uCalc.Item has a certain property.
    ///
    ///You can use the ItemOf function to cycle through the list of all items that share certain properties,
    ///or to disambiguate between items that share the same name but have different properties (for instance
    ///the + operator with the Prefix property vs another one with the Infix property).
    ///
    ///Some members are keywords in VB.NET.  To avoid naming conflict, the following members have been
    ///renamed in the VB.NET header: Function_, Operator_, Optional_, ByVal_, ByRef_, Class_, and NameSpace_.
    ///</remarks>
    enum ItemIsEnum : Int32 {
        AnyItem,
        LeftToRight,
        RightToLeft,
        DataType,
        Array,
        Class,
        NameSpace,
        Variable,
        LocalVariable,
        Function,
        Operator,
        FunctionOrOperator,
        Prefix,
        Infix,
        Postfix,
        AddressOwner,
        ByVal,
        ByRef,
        ByCopy,
        ByExpr,
        ByHandle,
        Pointer,
        Optional,
        VariableNumberOfArgs,
        RegEx,
        LiteralItem,
        Locked,
        Transform,
        StdCall,
        InTable,
        ItemDef,
        ErrorHandler,
        StopNow,
        CaseSensitive,
        QuotedText,
        ChangeMulti,
        ExpandArg,
        IgnoreStatementSep,
        IgnoreCodeBlock,
        SyntaxStop,
        SkipEvalExpand,
        ExprUnit,
        EagerArg,
        SkipSyntax,
        WhiteSpaceCounts,
        IgnoreQuote,
        UpToEndOrSeparator,
        UpToUserStop,
        EvalStrOutput,
        SkipOutput,
        DefaultSettings,
        NotFound,
        MiscItem
    };

    /// <summary>S: Licensing options for uCalc products</summary>
    enum LicenseTypeEnum : Int32 {
        License_Demo,
        License_FMP_Free,
        License_FMP_Standard_32,
        License_FMP_Standard_64,
        License_FMP_32_and_64,
        License_LanguageBuilder,
        License_GeneralParser,
        License_Interpreter,
        License_ConsoleCalc,
        License_WindowsCalc,
        License_StringLib,
        License_Transform,
        License_TransformLib,
        License_Search,
        License_SearchLib,
        License_Library1,
        License_Library2,
        License_Beta,
        License_Debugger,
        License_uCalc_All,
        License_Developer,
        License_EndUser
    };

    /// <summary>S: Types of tokens</summary>
    enum TokenEnum : Int32 {
        None,
        /// <summary> A token defined without any specified property is generic (default)</summary>
        Generic,
        /// <summary> Represents the absence of any character (token not found)</summary>
        Empty,
        /// <summary> A token that behaves as alphanuric (works with word boundaries)</summary>
        AlphaNumeric,
        /// <summary> A token that serves as argument separator (typically a comma)</summary>
        ArgSeparator,
        /// <summary> A token that starts a code block; typically characters like {, [, or (</summary>
        CodeBlock,
        /// <summary> The closing token that matches a CodeBlock token, such as }, ], or )</summary>
        CodeBlockMatch,
        /// <summary> A token that separates an object from its member, typically a . (a dot)</summary>
        MemberAccess,
        /// <summary> A token that represents a literal value, typically a quoted string</summary>
        Literal,
        /// <summary> A token that represents a statement separator, typically a ; (semicolon)</summary>
        StatementSep,
        /// <summary> A token that represents whitespace, typically tab or space characters</summary>
        WhiteSpace,
        /// <summary> A token that is not delimited by word boundaries (+, -, *, etc), consecutive characters are reduced until they match a value that exists in the symbol table or until 1 character</summary>
        Reducible,
        /// <summary> Non-character that represents the end of a stream of text</summary>
        Eof,
        CurrentThread, 
        LocalVar,
        StaticVar,
        /// <summary> Item in parenthesis (or quotes) following this token will be defined while the expression is parsed</summary>
        DefineNow = StaticVar,
        TempDef,
        /// <summary> The expression in parathesis (or quotes) following the token is evaluated and inserted in place of the token in the string</summary>
        EvalInsert,
        EvalDuringParse,
        /// <summary> Works like EvalInsert, except the expression is expanded instead of evaluated</summary>
        ExpandInsert,
        /// <summary> The contents of the quoted file name following this token is inserted into the string</summary>
        FileInclude,
        CurrentSpace,
        DefSpace,
        EvalSpace
    };

    /// <summary>S: Boolean True or False</summary>
    enum TrueFalseEnum : Int32 {
        /// <summary> For compilers in which False is a keyword use False_ instead</summary>
        False,
        /// <summary> For compilers in which True is a keyword use True_ instead</summary>
        True
    };

    /// <summary>S: Floating point environment exceptions</summary>
    enum FPEnvExceptionEnum : Int32 {
        /// <summary> Floating point division by zero, or operation resulting in infinity, occured (pole error)</summary>
        DivisionByZero = 8,
        /// <summary> The result is not an exact floating point number</summary>
        Inexact = 1,
        /// <summary> Argument passed to a function or operator was invalid (domain error)</summary>
        Invalid = 16,
        /// <summary> The result is too large to be represented as a floating point value of the return type (overflow range error)</summary>
        Overflow = 4,
        /// <summary> The result is too small to be represented as a floating point value of the return type, causing loss of precision (underflow range error)</summary>
        Underflow = 2,
        /// <summary> Represents selection of all the floating point exceptions: DivisionByZero, Inexact, Invalid, Overflow, and Underflow (Bitwise ORed)</summary>
        AllExceptions = 31
    };

    /// <summary>S: Floating point rouding options</summary>
    enum FPEnvRoundingEnum : Int32 {
        /// <summary> Rounds downward (towards negative infinity)</summary>
        Round_Down = 512,
        /// <summary> Rounds towards nearest value</summary>
        Round_ToNearest = 0,
        /// <summary> Rounds towards zero</summary>
        Round_TowardZero = 768,
        /// <summary> Rounds upward (towards positive infinity)</summary>
        Round_Up = 256
    };


    public delegate void uCalcDelegate(uCalcPtr Expr);
    public ref struct uCalc {
    public:
        static List<uCalcDelegate^>^ Addresses = gcnew List<uCalcDelegate^>();
        static uCalcDelegate ^ PinAddr(uCalcDelegate ^ Address) { Addresses->Add(Address); return Address; }
    private:

        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Evaluate")]
        static double Evaluate_(void *  ExprHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "EvaluateInt32")]
        static Int32 EvaluateInt32_(void *  ExprHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "EvaluateStr")]
        static STR_RETURN EvaluateStr_(void *  ExprHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "EvaluateVoid")]
        static void *   EvaluateVoid_(void *  ExprHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Execute")]
        static void Execute_(void *  ExprHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ReleaseExpr")]
        static void ReleaseExpr_(void *  ExprHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Handle_Overload_ParsedExpr")]
        static void *   Handle_Overload_ParsedExpr_(void *  ExprHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ElementCount")]
        static Int32 ElementCount_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "DataType_Overload_Item")]
        static uCalcPtr DataType_Overload_Item_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "FunctionAddress")]
        static uCalcCallback FunctionAddress_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "IsProperty")]
        static Int32 IsProperty_(void *  ItemHandle, ItemIsEnum ItemProperty);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Name_Overload_Item")]
        static STR_RETURN Name_Overload_Item_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "NextOverload")]
        static uCalcPtr NextOverload_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Precedence_Overload_Item")]
        static Int32 Precedence_Overload_Item_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ReleaseItem")]
        static void ReleaseItem_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ReplacementText")]
        static STR_RETURN ReplacementText_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "SetVariableValue")]
        static void SetVariableValue_(void *  ItemHandle, double Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "SetVariableValueAny")]
        static void SetVariableValueAny_(void *  ItemHandle, MARSHALSTR Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "SetVariableValueByte")]
        static void SetVariableValueByte_(void *  ItemHandle, SByte Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "SetVariableValueInt16")]
        static void SetVariableValueInt16_(void *  ItemHandle, Int16 Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "SetVariableValueInt32")]
        static void SetVariableValueInt32_(void *  ItemHandle, Int32 Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "SetVariableValueInt64")]
        static void SetVariableValueInt64_(void *  ItemHandle, Int64 Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "SetVariableValueSng")]
        static void SetVariableValueSng_(void *  ItemHandle, float Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "SetVariableValueDbl")]
        static void SetVariableValueDbl_(void *  ItemHandle, double Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "SetVariableValueStr")]
        static void SetVariableValueStr_(void *  ItemHandle, MARSHALSTR Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "SetVariableValuePtr")]
        static void SetVariableValuePtr_(void *  ItemHandle, void *   Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "SetVariableValueByPtr")]
        static void SetVariableValueByPtr_(void *  ItemHandle, void *   Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Text")]
        static STR_RETURN Text_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "uCalc_Overload_Item")]
        static uCalcPtr uCalc_Overload_Item_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ValueAddr")]
        static void *   ValueAddr_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Value")]
        static double Value_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ValueByte")]
        static SByte ValueByte_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ValueInt16")]
        static Int16 ValueInt16_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ValueInt32")]
        static Int32 ValueInt32_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ValueInt64")]
        static Int64 ValueInt64_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ValueSng")]
        static float ValueSng_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ValueDbl")]
        static double ValueDbl_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ValueStr")]
        static STR_RETURN ValueStr_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ValuePtr")]
        static void *   ValuePtr_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Handle_Overload_Item")]
        static void *   Handle_Overload_Item_(void *  ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Arg")]
        static double Arg_(void *  ExprPartHandle, Int32 Index);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ArgCount")]
        static Int32 ArgCount_(void *  ExprPartHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ArgObj")]
        static uCalcPtr ArgObj_(void *  ExprPartHandle, Int32 Index);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ArgAddr")]
        static void *   ArgAddr_(void *  ExprPartHandle, Int32 Index);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ArgExpr")]
        static uCalcPtr ArgExpr_(void *  ExprPartHandle, Int32 Index);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ArgInt32")]
        static Int32 ArgInt32_(void *  ExprPartHandle, Int32 Index);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ArgInt64")]
        static Int64 ArgInt64_(void *  ExprPartHandle, Int32 Index);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ArgPtr")]
        static void *   ArgPtr_(void *  ExprPartHandle, Int32 Index);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ArgStr")]
        static STR_RETURN ArgStr_(void *  ExprPartHandle, Int32 Index);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Return")]
        static void Return_(void *  ExprPartHandle, double Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ReturnInt32")]
        static void ReturnInt32_(void *  ExprPartHandle, Int32 Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ReturnInt64")]
        static void ReturnInt64_(void *  ExprPartHandle, Int64 Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ReturnPtr")]
        static void ReturnPtr_(void *  ExprPartHandle, void *   Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ReturnStr")]
        static void ReturnStr_(void *  ExprPartHandle, MARSHALSTR Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ErrorRaise")]
        static Int32 ErrorRaise_(void *  ExprPartHandle, ErrorNumberEnum ErrNumber);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ErrorRaiseMessage")]
        static Int32 ErrorRaiseMessage_(void *  ExprPartHandle, MARSHALSTR ErrMessage);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "uCalc_Overload_Callback")]
        static uCalcPtr uCalc_Overload_Callback_(void *  ExprPartHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Handle_Overload_Callback")]
        static void *   Handle_Overload_Callback_(void *  ExprPartHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "OpItem")]
        static uCalcPtr OpItem_(void *  ExprPartHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "AllocArray")]
        static void *   AllocArray_(void *  DataTypeHandle, Int32 ArraySize);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "AllocByPtr")]
        static void *   AllocByPtr_(void *  DataTypeHandle, void *   ValuePtr);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "AllocScalar")]
        static void *   AllocScalar_(void *  DataTypeHandle,  MARSHALSTR Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ByteSize")]
        static Int32 ByteSize_(void *  DataTypeHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Index")]
        static BuiltInTypesEnum Index_(void *  DataTypeHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Item_Overload_DataType")]
        static uCalcPtr Item_Overload_DataType_(void *  DataTypeHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Name_Overload_DataType")]
        static STR_RETURN Name_Overload_DataType_(void *  DataTypeHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ReleaseArray")]
        static void ReleaseArray_(void *  DataTypeHandle, void *   ValuePtr);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ReleaseScalar")]
        static void ReleaseScalar_(void *  DataTypeHandle, void *   ValuePtr);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ResetArray")]
        static void ResetArray_(void *  DataTypeHandle, void *   ValuePtr);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ResetScalar")]
        static void ResetScalar_(void *  DataTypeHandle, void *   ValuePtr);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "SetArray")]
        static void SetArray_(void *  DataTypeHandle, void *   Destination ,  void *   Source ,  Int32 Index);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "SetScalar")]
        static void SetScalar_(void *  DataTypeHandle, void *   Destination ,  void *   Source);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Swap")]
        static void Swap_(void *  DataTypeHandle, void *   ValuePtrA ,  void *   ValuePtrB);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ToString_Overload_DataType_ByPtr")]
        static STR_RETURN ToString_Overload_DataType_ByPtr_(void *  DataTypeHandle, void *   ValuePtr);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ToString_Overload_DataType_ByStr")]
        static STR_RETURN ToString_Overload_DataType_ByStr_(void *  DataTypeHandle, MARSHALSTR Value);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Handle_Overload_DataType")]
        static void *   Handle_Overload_DataType_(void *  DataTypeHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "AddErrorHandler")]
        static uCalcPtr AddErrorHandler_(void *  uCalcHandle, UCCALLBACK ErrorHandlerAddress ,  Int32 Position);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "CreateuCalc")]
        static uCalcPtr CreateuCalc_(void *  uCalcHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "DataTypeOfIndex")]
        static uCalcPtr DataTypeOfIndex_(void *  uCalcHandle, BuiltInTypesEnum Index);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Define")]
        static uCalcPtr Define_(void *  uCalcHandle, MARSHALSTR Expression  ,  void *   Address  ,  void *   ExistingItem);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "DefineConstant")]
        static uCalcPtr DefineConstant_(void *  uCalcHandle, MARSHALSTR Definition  ,  void *   VariableAddress);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "DefineDataType")]
        static uCalcPtr DefineDataType_(void *  uCalcHandle, Int32 TypeSize ,  void *   AllocArrayValue ,  void *   AllocScalar ,  void *   AllocScalarByPtr ,  void *   DeleteArray ,  void *   DeleteScalar ,  void *   ResetScalar ,  void *   ResetArray ,  void *   SetValueArray ,  void *   SetValueScalar ,  void *   SwapValues ,  void *   ToString);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "DefineFunction")]
        static uCalcPtr DefineFunction_(void *  uCalcHandle, MARSHALSTR Definition  ,  UCCALLBACK FunctionAddress);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "DefineOperator")]
        static uCalcPtr DefineOperator_(void *  uCalcHandle, MARSHALSTR Definition  ,  Int32 iPrecedence ,  GroupingEnum Grouping  ,  UCCALLBACK FunctionAddress);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "DefineSyntax")]
        static uCalcPtr DefineSyntax_(void *  uCalcHandle, MARSHALSTR Definition);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "DefineTransform")]
        static uCalcPtr DefineTransform_(void *  uCalcHandle, MARSHALSTR FromExpr  ,  MARSHALSTR ToExpr  ,  void *   ItemHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "DefineVariable")]
        static uCalcPtr DefineVariable_(void *  uCalcHandle, MARSHALSTR Definition  ,  void *   VariableAddress);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ErrorExpression")]
        static STR_RETURN ErrorExpression_(void *  uCalcHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ErrorLocation")]
        static Int32 ErrorLocation_(void *  uCalcHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ErrorMessage")]
        static STR_RETURN ErrorMessage_(void *  uCalcHandle,  ErrorNumberEnum Index);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ErrorNumber")]
        static ErrorNumberEnum ErrorNumber_(void *  uCalcHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ErrorResponse")]
        static void ErrorResponse_(void *  uCalcHandle, ErrorHandlerResponseEnum Response);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ErrorSymbol")]
        static STR_RETURN ErrorSymbol_(void *  uCalcHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ErrorSetExpression")]
        static void ErrorSetExpression_(void *  uCalcHandle, MARSHALSTR Expression);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Eval")]
        static double Eval_(void *  uCalcHandle, MARSHALSTR Expression);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "EvalStr")]
        static STR_RETURN EvalStr_(void *  uCalcHandle, MARSHALSTR Expression);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Expand")]
        static STR_RETURN Expand_(void *  uCalcHandle, MARSHALSTR Expression);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ExpandSteps")]
        static STR_RETURN ExpandSteps_(void *  uCalcHandle, MARSHALSTR Expression  ,  MARSHALSTR Separator);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "FPEnvClearException")]
        static Int32 FPEnvClearException_(void *  uCalcHandle,  Int32 Exceptions);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "FPEnvGetExceptionMask")]
        static Int32 FPEnvGetExceptionMask_(void *  uCalcHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "FPEnvRaiseException")]
        static Int32 FPEnvRaiseException_(void *  uCalcHandle,  FPEnvExceptionEnum Exception);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "FPEnvGetRounding")]
        static FPEnvRoundingEnum FPEnvGetRounding_(void *  uCalcHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "FPEnvSetExceptionMask")]
        static void FPEnvSetExceptionMask_(void *  uCalcHandle, Int32 Exceptions);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "FPEnvSetExceptionUnMask")]
        static void FPEnvSetExceptionUnMask_(void *  uCalcHandle, Int32 Exceptions);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "FPEnvSetRounding")]
        static Int32 FPEnvSetRounding_(void *  uCalcHandle, FPEnvRoundingEnum Direction);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "FPEnvToggleExceptionMask")]
        static void FPEnvToggleExceptionMask_(void *  uCalcHandle, FPEnvExceptionEnum Exception);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "HandleOf")]
        static uCalcPtr HandleOf_(void *  uCalcHandle, MARSHALSTR ItemName ,   Int64 Properties ,   bool SkipExpansion);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ItemOf_Overload_uCalc_ByName")]
        static uCalcPtr ItemOf_Overload_uCalc_ByName_(void *  uCalcHandle, MARSHALSTR ItemName  ,  Int64 Properties  ,  bool SkipExpansion);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ItemOf_Overload_uCalc_ByProperty")]
        static uCalcPtr ItemOf_Overload_uCalc_ByProperty_(void *  uCalcHandle, ItemIsEnum Property_ ,  Int32 nth ,  MARSHALSTR DataTypeName);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "License")]
        static Int32 License_(void *  uCalcHandle, LicenseTypeEnum LicenseType,   Int32 L1 ,   Int32 L2 ,   Int32 L3 ,   Int32 L4 ,   Int32 L5);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "NamespaceChild")]
        static uCalcPtr NamespaceChild_(void *  uCalcHandle,  Int32 nth);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "NamespaceChildCount")]
        static Int32 NamespaceChildCount_(void *  uCalcHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "NamespaceParent")]
        static uCalcPtr NamespaceParent_(void *  uCalcHandle,  Int32 Level);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "NamespaceSymbol")]
        static uCalcPtr NamespaceSymbol_(void *  uCalcHandle, Int32 nth);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "NamespaceSymbolCount")]
        static Int32 NamespaceSymbolCount_(void *  uCalcHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Parse")]
        static uCalcPtr Parse_(void *  uCalcHandle, MARSHALSTR Expression  ,  MARSHALSTR DataType);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Precedence_Overload_uCalc")]
        static Int32 Precedence_Overload_uCalc_(void *  uCalcHandle, MARSHALSTR Operator1 ,   MARSHALSTR Operator2);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Properties")]
        static Int64 Properties_(void *  uCalcHandle, ItemIsEnum p1,   ItemIsEnum p2 ,   ItemIsEnum p3 ,   ItemIsEnum p4);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ReleaseuCalc")]
        static void ReleaseuCalc_(void *  uCalcHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ResetOutput")]
        static void ResetOutput_(void *  uCalcHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Default")]
        static uCalcPtr Default_(void *  uCalcHandle);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "SetOutput")]
        static uCalcPtr SetOutput_(void *  uCalcHandle, UCCALLBACK FunctionAddress,  MARSHALSTR Definition);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "ValueAt")]
        static STR_RETURN ValueAt_(void *  uCalcHandle, void *   ValuePtr,  MARSHALSTR DataType);
        [DllImport(uCalcDLL, CharSet=CharSet::Ansi, CallingConvention=CallingConvention::Cdecl, EntryPoint = "Handle_Overload_uCalc")]
        static void *   Handle_Overload_uCalc_(void *  uCalcHandle);



    public:
        ref struct ParsedExpr;
        ref struct Item;
        ref struct Callback;
        ref struct DataType;




        /// <summary> uCalc main class</summary>


        uCalcPtr uCalcHandle;
        /// <summary> Handle for uCalc instance</summary>


        ref struct ParsedExpr { 
            /// <summary> Class for evaluating an already parsed expression or executing parsed code</summary>


            uCalcPtr ExprHandle;





            /// <summary> Rapidly evaluates a floating point expression that was parsed with Parse or passed By Expression</summary>
            /// <returns> The result of the evaluated floating point expression</returns>
            /// <remarks><para>Evaluate can be used instead of Eval for faster results in time-critical loops where the same expression
            /// is repeatedly re-evaluated, with the only change in each iteration typically being the value(s) of one or
            /// more variables.  Evaluate works in pair with Parse, the relatively slower part, which is called first before
            /// going into the time-critical loop.  The ParsedExpr object returned by Parse is what calls the Evaluate function.
            /// </para><para>
            /// Evaluate can also be called by a ParsedExpr object that was passed to a callback By Expression.
            /// </para><para>
            /// The expression must be one who&#39;s evaluation returns a floating point numeric value.  If the expression evaluates
            /// to a non-numeric value (such as a string) or non-scalar numeric value (such as a complex number), or if a numeric
            /// type other than the default was explicitely specified with Parse, the value returned will be invalid.  However,
            /// an error will not be raised.
            /// </para></remarks> 
            double Evaluate() { return  Evaluate_(ExprHandle); }





            /// <summary> Rapidly evaluates expression that was parsed as integer with Parse or passed By Expression</summary>
            /// <returns> The result of the evaluated integer expression</returns>
            /// <remarks><para>
            /// This is similar to Evaluate, but applies only to integer expressions.
            /// </para></remarks> 
            Int32 EvaluateInt32() { return  EvaluateInt32_(ExprHandle); }





            /// <summary> Rapidly evaluates an expression of any data type that was parsed with Parse or passed By Expression returning a string</summary>
            /// <returns> The result of an evaluated expression of any data type, returned as a string</returns>
            /// <remarks><para>EvaluateStr is similar to Evaluate.  However, EvaluateStr returns a value as a string.  Unlike Evaluate, which can only
            /// return a floating point value, EvaluateStr can return a value of any data type.  Non-string values are converted to a string
            /// before being returned.
            /// </para></remarks> 
            STRING EvaluateStr() { RETURNSTRING(EvaluateStr_(ExprHandle)); }





            /// <summary> Rapidly evaluates an expression that was parsed with Parse or passed By Expression and returns a void *   to the result</summary>
            /// <returns> Returns a void *   to the result instead of the result itself</returns>
            /// <remarks><para>Use this when you are interested in a void *   to the result instead of the result itself.  For instance, given an integer
            /// result, you might use a void *   in order to retrieve the hi SByte and lo SByte values.
            /// </para></remarks> 
            void *   EvaluateVoid() { return  EvaluateVoid_(ExprHandle); }





            /// <summary> Rapidly executes code that was parsed with Parse or passed By Expression for which a result is not required</summary>
            /// <remarks><para>This can be used in cases where the reutrn value is not needed, such as statements that represent code to be executed
            /// rather than a math expression.
            /// </para></remarks> 
            void Execute() {  Execute_(ExprHandle); }





            /// <summary> (Use Release instead)</summary>
            /// <remarks><para>Use Release</para></remarks> 
            void ReleaseExpr() {  ReleaseExpr_(ExprHandle); }





            /// <summary> Returns the handle of the parsed expression</summary>
            /// <remarks><para>This numeric value represents the void *   that uCalc uses internally for the ParsedExpr object.
            /// If you are iterating through a series of objects, such as with NextOverload, you may check Handle()
            /// to see when it becomes NULL (or 0) to know when to stop.</para></remarks>
            void *    Handle() { return  Handle_Overload_ParsedExpr_(ExprHandle); }




            void Release() { ReleaseExpr(); ExprHandle = 0; }
            /// <summary> Releases an expression that was parsed with Parse</summary>
            /// <remarks><para>Whenever an expression is parsed with Parse, the parsed expression remains in memory until released.
            /// When the parsed expression is no longer needed it should be released with Release to preserve memory.
            /// Note: An expression parsed with Parse must explicitely be released with Release unless it the uCalc
            /// object it is in is released.  A parsed expression will not be released automatically when it simply
            /// goes out of scope.
            /// </para></remarks> 


            ParsedExpr() { ExprHandle = 0; }
            ParsedExpr( uCalcPtr ExpressionHandle) { ExprHandle = ExpressionHandle; }
            ParsedExpr(ParsedExpr % ParsedExprInstance) { ExprHandle = ParsedExprInstance.ExprHandle; }
            ParsedExpr % operator= (ParsedExpr % ParsedExprInstance) { ExprHandle = ParsedExprInstance.ExprHandle; return *this; }  
        };


        ref struct Item { 
            /// <summary> Class for handling uCalc items</summary>


            uCalcPtr ItemHandle;
            /// <summary> Handle for a uCalc.Item</summary>



            /// <summary> Returns the number of elements in a uCalc.Item</summary>
            /// <remarks><para>Depending on the kind of object, this may represent the number of elements in an array, the number of parameters in a function,
            /// the number of operands in an operator, the number of parameters in a syntax construct, the number of parts in a transform,
            /// the number of parts in a compound token, or the value explicitlely set in the Count property when calling Define.
            /// </para><para>
            /// When determining the number of elements, optional parameters in a function count as an element.  If the function
            /// has a variable number of arguments, then count is -1.
            /// </para><para>
            /// The value returned for a syntax construct may not seem obvious, especially if it has optional and/or alternative
            /// parts or some other complexity.
            /// </para><para>
            /// +++ the number of symbols in a uCalc
            /// instance,</para></remarks>
            Int32 ElementCount() { return  ElementCount_(ItemHandle); }





            /// <summary> Returns the data type of a uCalc.Item</summary>
            /// <remarks><para>The returned data type object can be used to determine the type name, size, etc.</para></remarks>
            DataType  DataType() { uCalc::DataType TempObj(DataType_Overload_Item_(ItemHandle)); return TempObj; }





            /// <summary> Returns the callback address of a defined function (or operator)</summary>
            /// <remarks><para>The returned value is the actual void *   address of the function as used by uCalc internally, and not
            /// a UCCALLBACK, even in .NET.  As such, in .NET this value can be used in the more general Define function but
            /// not functions like DefineFunction or DefineneOperator.</para></remarks>
            uCalcCallback FunctionAddress() { return  FunctionAddress_(ItemHandle); }





            /// <summary> Determines if a uCalc.Item has the given properties</summary>
            /// <param name="ItemProperty"> The property you want to check for</param>
            /// <returns> Returns True if the uCalc.Item matches the given property, or False otherwise.</returns>
            /// <remarks><para>This helps you determine whether the uCalc.Item is a function, operator, variable, data type,
            /// literal value, etc, or has certain other special properties.  Note: Some items may have multiple
            /// properties.  For instance the Function property may be set to true for arrays.
            /// </para></remarks>
            bool IsProperty(ItemIsEnum ItemProperty) { return IsProperty_(ItemHandle,ItemProperty) != 0; }





            /// <summary> Returns the name of the uCalc.Item</summary>
            /// <remarks><para>Items such as functions, operators, variables, etc generally are given a name when they are defined.
            /// That name can be retried with this function.</para></remarks>
            STRING  Name() { RETURNSTRING(Name_Overload_Item_(ItemHandle)); }





            /// <summary> Returns next uCalc.Item object that shares the same name</summary>
            /// <returns> Returns next uCalc.Item object that shares the same name, or a NULL (or 0) if there is none</returns>
            /// <remarks><para>If several functions, operators, parameters, etc are overloaded -- that is they are defined with different
            /// parameters but share the same name --, or if they (or variables) overshadow each other, this function returns
            /// the next uCalc.Item object that shares the same name.</para></remarks>
            Item NextOverload() { uCalc::Item TempObj(NextOverload_(ItemHandle)); return TempObj; }





            /// <summary> Returns the precedence level of the uCalc.Item</summary>
            /// <remarks><para>Operators, and some syntax constructs are defined with a precedence level.  When defining a new operator you
            /// may sometimes want to give it the same level as a pre-existing operator.</para></remarks> 
            Int32  Precedence() { return  Precedence_Overload_Item_(ItemHandle); }





            /// <summary> (Use Release instead)</summary>
            /// <remarks><para>Use Release</para></remarks> 
            void ReleaseItem() {  ReleaseItem_(ItemHandle); }





            /// <summary> Returns the replacement part of a syntax construct defined with DefineSyntax</summary>
            /// <returns> Returns the part to the right of &quot;::=&quot; in a syntax construct defined with DefineSyntax</returns>
            /// <remarks><para>This allows you to inspect the part to the right of &quot;::=&quot; in a syntax construct defined with DefineSyntax.</para></remarks> 
            STRING ReplacementText() { RETURNSTRING(ReplacementText_(ItemHandle)); }





            /// <summary> Sets the value of a floating point variable +++ maybe SetValue instead of SetVariableValue</summary>
            /// <param name="Value"> Double precision floating point value that the variable will be set to</param>
            /// <remarks><para>When a floating point variable is defined with DefineVariable, you can later set a value for this variable using
            /// SetVariableValue.  Note: If your compiler supports pointers, you can define a uCalc variable that is directly attached
            /// to the address of a variable in your source code, and not have to use SetVariableValue.</para></remarks> 
            void SetVariableValue( double Value) {  SetVariableValue_(ItemHandle,Value); }





            /// <summary> Sets the value of a variable of any data type by passing a string</summary>
            /// <param name="Value"> String containing the value that will be converted and stored in the variable</param>
            /// <remarks><para>This lets you set the value of a variable defined with DefineVariable (or Define) regardless of the variable&#39;s data type.
            /// The value is passed as a string, and internally converted to the appropriate data type.  Note: Because a conversion takes
            /// place with SetVariableValueAny, it is typically slower than calling SetVariableValue for floating point, or one of the
            /// other versions of SetVariableValue specific to a data type.</para></remarks> 
            void SetVariableValueAny( CONSTSTRING Value ) {  SetVariableValueAny_(ItemHandle,Value); }






            /// <summary> Sets the value of a SByte variable</summary>
            /// <param name="Value"> SByte value that the variable will be set to</param>
            /// <remarks><para>This lets you set the value of a SByte variable defined with DefineVariable (or Define)</para></remarks> 
            void SetVariableValueByte( SByte Value) {  SetVariableValueByte_(ItemHandle,Value); }





            /// <summary> Sets the value of a 16-bit integer variable</summary>
            /// <param name="Value"> 16-bit integer value that the variable will be set to</param>
            /// <remarks><para>This lets you set the value of a 16-bit integer variable defined with DefineVariable (or Define).  This works for
            /// both signed and unsigned 16-bit integers.</para></remarks> 
            void SetVariableValueInt16( Int16 Value) {  SetVariableValueInt16_(ItemHandle,Value); }





            /// <summary> Sets the value of a 32-bit integer variable</summary>
            /// <param name="Value"> 32-bit integer value that the variable will be set to</param>
            /// <remarks><para>This lets you set the value of a 32-bit integer variable defined with DefineVariable (or Define).  This works for
            /// both signed and unsigned 32-bit integers.</para></remarks> 
            void SetVariableValueInt32( Int32 Value) {  SetVariableValueInt32_(ItemHandle,Value); }





            /// <summary> Sets the value of a 64-bit integer variable</summary>
            /// <param name="Value"> 64-bit integer value that the variable will be set to</param>
            /// <remarks><para>This lets you set the value of a 64-bit integer variable defined with DefineVariable (or Define).  This works for
            /// both signed and unsigned 64-bit integers.</para></remarks> 
            void SetVariableValueInt64( Int64 Value) {  SetVariableValueInt64_(ItemHandle,Value); }





            /// <summary> Sets the value of a single precision floating point variable</summary>
            /// <param name="Value"> Single precision value that the variable will be set to</param>
            /// <remarks><para>This lets you set the value of a single precision variable defined with DefineVariable (or Define).</para></remarks> 
            void SetVariableValueSng( float Value) {  SetVariableValueSng_(ItemHandle,Value); }





            /// <summary> Sets the value of a double precision floating point variable</summary>
            /// <param name="Value"> Double precision floating point value that the variable will be set to</param>
            /// <remarks><para>SetVariableValueDbl is synonymous with SetVariableValue</para></remarks> 
            void SetVariableValueDbl( double Value) {  SetVariableValueDbl_(ItemHandle,Value); }





            /// <summary> Sets the value of a string variable</summary>
            /// <param name="Value"> String value that the variable will be set to</param>
            /// <remarks><para>This lets you set the value of a string variable defined with DefineVariable (or Define).</para></remarks> 
            void SetVariableValueStr( CONSTSTRING Value ) {  SetVariableValueStr_(ItemHandle,Value); }






            /// <summary> Sets the value of a void *   variable</summary>
            /// <param name="Value"> void *   that the variable will be set to</param>
            /// <remarks><para>This lets you set the value of a void *   variable defined with DefineVariable (or Define).  Note: This does not
            /// change the void *   of the variable.  It simply stores a void *   value in the variable.</para></remarks> 
            void SetVariableValuePtr( void *   Value) {  SetVariableValuePtr_(ItemHandle,Value); }





            /// <summary> Sets the value of a variable to a value who&#39;s void *   is passed</summary>
            /// <param name="Value"> void *   to the value that the variable will be set to</param>
            /// <remarks><para>This lets you set the value of a variable by passing the void *   to the value you want to set the variable to.</para></remarks> 
            void SetVariableValueByPtr( void *   Value) {  SetVariableValueByPtr_(ItemHandle,Value); }





            /// <summary> Returns the text data associated with a uCalc.Item</summary>
            /// <remarks><para>Certain items, such as a syntax construct defined with DefineSyntax have a text property that can be retrieved
            /// with a call to this function.</para></remarks> 
            STRING Text() { RETURNSTRING(Text_(ItemHandle)); }





            /// <summary> Returns the uCalc object that the uCalc.Item belongs to</summary>
            /// <remarks><para>A uCalc.Item is defined in the context of a uCalc object.  You can later use this function to determine
            /// the uCalc object the defined item belongs to.</para></remarks> 
            uCalc  uCalc() { uCalcNameSpace::uCalc TempObj(uCalc_Overload_Item_(ItemHandle)); return TempObj; }





            /// <summary> Returns the void *   address of the value associated with a uCalc.Item</summary>
            /// <remarks><para>Use this to retrieve the void *   address of the value associated with a variable (instead of
            /// the value itself).  Note: If the uCalc.Item is a Variable of type void *  , then ValuePtr returns a void *   value
            /// that is stored in that variable, whereas ValueAddr returns the void *   of the variable itself.  In other words
            /// ValueAddr returns the address in memory of the current uCalc.Item value, while ValuePtr returns any given address
            /// you may have stored as the value of the variable.</para></remarks> 
            void *   ValueAddr() { return  ValueAddr_(ItemHandle); }





            /// <summary> Returns the value (default double precision) associated with a uCalc.Item</summary>
            /// <remarks><para>Use this to retrieve the Double precision value associated with a variable.</para></remarks> 
            double Value() { return  Value_(ItemHandle); }





            /// <summary> Returns the value (default double precision) associated with a uCalc.Item</summary>
            /// <remarks><para>Use this to retrieve the SByte value associated with a variable.</para></remarks> 
            SByte ValueByte() { return  ValueByte_(ItemHandle); }





            /// <summary> Returns the 16-bit integer value associated with a uCalc.Item</summary>
            /// <remarks><para>Use this to retrieve the Int16 value associated with a variable.</para></remarks> 
            Int16 ValueInt16() { return  ValueInt16_(ItemHandle); }





            /// <summary> Returns the 32-bit integer value associated with a uCalc.Item</summary>
            /// <remarks><para>Use this to retrieve the Int32 value associated with a variable.</para></remarks> 
            Int32 ValueInt32() { return  ValueInt32_(ItemHandle); }





            /// <summary> Returns the 64-bit integer value associated with a uCalc.Item</summary>
            /// <remarks><para>Use this to retrieve the Int64 value associated with a variable.</para></remarks> 
            Int64 ValueInt64() { return  ValueInt64_(ItemHandle); }





            /// <summary> Returns the Double precision value associated with a uCalc.Item</summary>
            /// <remarks><para>Use this to retrieve the Single precision value associated with a variable.</para></remarks> 
            float ValueSng() { return  ValueSng_(ItemHandle); }





            /// <summary> Returns the Double precision value associated with a uCalc.Item</summary>
            /// <remarks><para>Synonymous with Value</para></remarks> 
            double ValueDbl() { return  ValueDbl_(ItemHandle); }





            /// <summary> Returns the String value associated with a uCalc.Item</summary>
            /// <remarks><para>Use this to retrieve the String value associated with a variable.
            /// </para><para>
            /// ValueStr has a special property beyond the Value function for other data types in that it can return the value
            /// of an item regardless of what data type it has.  The value is returned as a string.
            /// </para></remarks> 
            STRING ValueStr() { RETURNSTRING(ValueStr_(ItemHandle)); }





            /// <summary> Returns the void *   value associated with a uCalc.Item</summary>
            /// <remarks><para>Use this to retrieve the void *   value associated with a variable.
            /// Note: If the uCalc.Item is a Variable of type void *  , then ValuePtr returns a void *   value
            /// that is stored in that variable, whereas ValueAddr returns the void *   of the variable itself.  In other words
            /// ValueAddr returns the address in memory of the current uCalc.Item value, while ValuePtr returns any given address
            /// you may have stored as the value of the variable.</para></remarks> 
            void *   ValuePtr() { return  ValuePtr_(ItemHandle); }





            /// <summary> Returns the handle of the current uCalc.Item object</summary>
            void *    Handle() { return  Handle_Overload_Item_(ItemHandle); }




            void Release() { ReleaseItem(); ItemHandle = 0; }
            /// <summary> Releases an Item defined with Define or other related function</summary>
            /// <remarks><para>When a function, operator, variable, or other item is defined with such functions as
            /// Define, DefineFunction, DefineOperator, DefineVariable, these items remain in memory, until they are
            /// released with this function.  Items should generally be released when they are no longer needed, in order
            /// to preserve memory.
            /// </para><para>
            /// Note: A uCalc Item object must explicitely be released with Release unless the entire uCalc
            /// object it is a part of is is released.  An Item object will not be released automatically when it
            /// simply goes out of scope.
            /// </para></remarks> 


            Item() { ItemHandle = 0; }
            Item( uCalcPtr PassedItemHandle) { ItemHandle = PassedItemHandle; }
            Item(Item % ItemHandleInstance) { ItemHandle = ItemHandleInstance.ItemHandle; }
            Item % operator= (Item % ItemHandleInstance) { ItemHandle = ItemHandleInstance.ItemHandle; return *this; }  
        };




        ref struct Callback { 
            /// <summary> Class for retrieving arguments passed to a callback, returning a value, and more</summary>


            uCalcPtr ExprPartHandle;
            /// <summary> Handle of the expression part the current callback instance is refering to</summary>



            /// <summary> Returns the value of an argument passed to a uCalc.Callback function</summary>
            /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
            /// <remarks><para>When the name of a callback function is specified in DefineFunction or DefineOperator, that
            /// callback function will receive arguments uCalc passes to it through the Arg.  Arg handles values
            /// of the default Double floating point data type.  For other data types, ArgInt32, ArgStr, etc instead.
            /// </para><para>
            /// Note that Index is 1-based.  So if there are 3 arguments, the Index of the first will be
            /// 1 (not 0), and the second and third will be 2 and 3 respectively.
            /// </para><para>
            /// If you are defining a binary operator, 1 is the index for the left operand, and 2 for the right operand.
            /// For a unary operator, the index is 1.
            /// </para></remarks> 
            double Arg( Int32 Index) { return  Arg_(ExprPartHandle,Index); }





            /// <summary> Returns the number of arguments being passed to a uCalc.Callback function</summary>
            /// <returns> 0 if no arg is passed; or a positive integer representing the number of args being passed.</returns>
            /// <remarks><para>This is useful for functions that have a variable number of arguments</para></remarks> 
            Int32 ArgCount() { return  ArgCount_(ExprPartHandle); }





            /// <summary> Returns an object instance of the argument being passed to a uCalc.Callback function</summary>
            /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
            /// <returns> The returned object can be used (where relevent) just like any other uCalc.Item</returns>
            /// <remarks><para>Each argument is a uCalc.Item, just like anything else defined with Define.  With this object,
            /// you can do such things as determine the argument&#39;s data type, determine which instance of uCalc
            /// it belongs to, retrieve the void *   of the value of the argument, etc.</para></remarks> 
            Item ArgObj( Int32 Index) { uCalc::Item TempObj(ArgObj_(ExprPartHandle,Index)); return TempObj; }





            /// <summary> Returns the address of the argument value that is being passed</summary>
            /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
            /// <remarks><para>In addition to Arg which returns arguments that are of the default Double floating point numeric type, there
            /// are specialized functions for other datatypes such as ArgInt32, ArgInt64, etc.  However, if the argument is of
            /// a data type that does not have a specialized function, you can retrieve the address of the argument value as
            /// a void *  , and use that to retrieve the value.</para></remarks>  
            void *   ArgAddr( Int32 Index) { return  ArgAddr_(ExprPartHandle,Index); }





            /// <summary> Returns the object instance of the expression that&#39;s passed as an argument to a uCalc.Callback function+++Example</summary>
            /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
            /// <returns> Returns parsed expression object with which you can call functions like Evaluate or Execute</returns>
            /// <remarks><para>uCalc allows you to not only pass arguments by value or by reference.  But it also lets you pass an argument
            /// by expression.  When an expression is passed By Expression, the expresison is not evaluated before being passed.
            /// Instead a handle to the parsed expression is passed, allowing the callback to perform the evaluation.</para></remarks> 
            ParsedExpr ArgExpr( Int32 Index) { uCalc::ParsedExpr TempObj(ArgExpr_(ExprPartHandle,Index)); return TempObj; }





            /// <summary> Returns the Int32 integer argument passed to a uCalc.Callback function</summary>
            /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
            /// <remarks><para>If a function parameter (or operator operand) was explicitely defined as Int32, then the callback function can
            /// recieve it through ArgInt32.  If the argument is of a different but compatible numeric type, a conversion takes
            /// place first before being passed.</para></remarks> 
            Int32 ArgInt32( Int32 Index) { return  ArgInt32_(ExprPartHandle,Index); }





            /// <summary> Returns the Int64 integer argument being passed to a uCalc.Callback function</summary>
            /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
            /// <remarks><para>If a function parameter (or operator operand) was explicitely defined as Int64, then the callback function can
            /// recieve it through ArgInt64.  If the argument is of a different but compatible numeric type, a conversion takes
            /// place first before being passed.</para></remarks> 
            Int64 ArgInt64( Int32 Index) { return  ArgInt64_(ExprPartHandle,Index); }





            /// <summary> Returns a void *   value being passed as an argument to a uCalc.Callback function</summary>
            /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
            /// <remarks><para>If a function parameter (or operator operand) was explicitely defined as a void *  , then the callback function can
            /// recieve it through ArgPtr.  Note that ArgPtr returns any address value that is being sent, whereas ArgAddr returns the
            /// address of the value being passed.</para></remarks> 
            void *   ArgPtr( Int32 Index) { return  ArgPtr_(ExprPartHandle,Index); }





            /// <summary> Returns the value being passed as argument in to a uCalc.Callback function in form of a string</summary>
            /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
            /// <remarks><para>In addition to returning an argument that was passed as a string, ArgStr has the special property of returning an argument that
            /// was passed as any data type.  If the argument was not a string, ArgStr converts it to a string before returning its value.</para></remarks> 
            STRING ArgStr( Int32 Index) { RETURNSTRING(ArgStr_(ExprPartHandle,Index)); }





            /// <summary> Sets the return value (default floating point) of a uCalc.Callback function</summary>
            /// <param name="Value"> Floating point value that your callback returns</param>
            /// <remarks><para>When you define a callback function with DefineFunction and you want to return a floating point number,
            /// do it with Return.
            /// </para><para>
            /// Note: For compilers in which Return is a reserved keyword, use ReturnDbl instead.
            /// </para></remarks> 
            void Return( double Value) {  Return_(ExprPartHandle,Value); }





            /// <summary> Sets the return value of a uCalc.Callback function as an Int32 integer</summary>
            /// <param name="Value"> Integer (32-bit) that your callback returns</param>
            /// <remarks><para>When you define a callback function with DefineFunction and you want to return a Int32,
            /// do it with ReturnInt32.</para></remarks> 
            void ReturnInt32( Int32 Value) {  ReturnInt32_(ExprPartHandle,Value); }





            /// <summary> Sets the return value of a uCalc.Callback function as an Int64 integer</summary>
            /// <param name="Value"> Integer (64-bit) that your callback returns</param>
            /// <remarks><para>When you define a callback function with DefineFunction and you want to return a Int64,
            /// do it with ReturnInt64.</para></remarks> 
            void ReturnInt64( Int64 Value) {  ReturnInt64_(ExprPartHandle,Value); }





            /// <summary> Sets the return value of a uCalc.Callback function as a void *  </summary>
            /// <param name="Value"> void *   value that your callback returns</param>
            /// <remarks><para>When you define a callback function with DefineFunction that returns a void *   value,
            /// do it with ReturnPtr.</para></remarks> 
            void ReturnPtr( void *   Value) {  ReturnPtr_(ExprPartHandle,Value); }





            /// <summary> Sets the return value of a uCalc.Callback function as a String</summary>
            /// <param name="Value"> String value that your callback returns</param>
            /// <remarks><para>When you define a callback function with DefineFunction that returns a string,
            /// do it with ReturnStr.</para></remarks> 
            void ReturnStr( CONSTSTRING Value ) {  ReturnStr_(ExprPartHandle,Value); }






            /// <summary> Raises an error in a uCalc.Callback function</summary>
            /// <param name="ErrNumber"> Index number of error to raise</param>
            /// <returns> Returns ErrorAbort, ErrorResume, or ErrorReRaise.</returns>
            /// <remarks><para>Your callback can raise an error by calling this funciton with one of the values of the
            /// ErrorNumberEnum enumerator.  End-user function definitions can also raise an error by calling Error().
            /// The return value is determined by error handlers that you may have added.  ErrorAbort (value of 0)
            /// is returned by default if there are no error handlers.</para></remarks>
            Int32 ErrorRaise( ErrorNumberEnum ErrNumber) { return  ErrorRaise_(ExprPartHandle,ErrNumber); }





            /// <summary> Raises an error with a user-defined message in a uCalc.Callback function</summary>
            /// <param name="ErrMessage"> Text of custom error message to return</param>
            /// <returns> Returns ErrorAbort, ErrorResume, or ErrorReRaise.</returns>
            /// <remarks><para>Your callback can raise a custom error message (one not listed as a ErrorNumberEnum enumerator)
            /// by simply passing the text of the message to this function.</para></remarks>
            Int32 ErrorRaiseMessage( CONSTSTRING ErrMessage ) { return  ErrorRaiseMessage_(ExprPartHandle,ErrMessage); }






            /// <summary> Returns the uCalc instance the current expression part belongs to</summary>
            /// <remarks><para>A callback is designed to work with all instances of uCalc.  However, at times you may need to know
            /// which particular instance of uCalc the expression part being passed belongs to.</para></remarks>
            uCalc  uCalc() { uCalcNameSpace::uCalc TempObj(uCalc_Overload_Callback_(ExprPartHandle)); return TempObj; }





            /// <summary> Returns the handle of the sub expression passed to the callback</summary>
            /// <remarks><para>This void *   value is the argument received by a callback function associated with a function or operator
            /// defined with DefineFunction or DefineOperator.  Once a uCalc.Callback object is construct with this handle, this
            /// handle typically no longer needs to be refered to directly.  However, it&#39;s still avaible should you need it.
            /// uCalc interfaces with this handle underneath the hood of the uCalc.Callback object.
            /// </para></remarks>
            void *    Handle() { return  Handle_Overload_Callback_(ExprPartHandle); }





            /// <summary> Returns the function or operator Item of the current expression part</summary>
            /// <remarks><para>If several functions and/or operators are mapped to the same callback function, you can use this to
            /// let you determine which one is currently being called, based on properties you can check.</para></remarks>
            Item OpItem() { uCalc::Item TempObj(OpItem_(ExprPartHandle)); return TempObj; }




            Callback() { ExprPartHandle = 0; }
            Callback( uCalcPtr ExpressionHandle) { ExprPartHandle = ExpressionHandle; }
            Callback(Callback % ExprInstance) { ExprPartHandle = ExprInstance.ExprPartHandle; }
            Callback % operator= (Callback % ExprInstance) { ExprPartHandle = ExprInstance.ExprPartHandle; return *this; }  
        };


        ref struct DataType { 
            /// <summary> Class for working with uCalc data types</summary>


            uCalcPtr DataTypeHandle;
            /// <summary> Handle for uCalc Data Type</summary>



            /// <summary> Allocates an array of the given data type</summary>
            /// <param name="ArraySize"> Size - in umber of elements - of the array you&#39;re allocating</param>
            /// <returns> void *   of the array that was allocated</returns>
            /// <remarks><para>This allocates an array of the given type.  Elements of the array are set to the default value
            /// of that type.</para></remarks>
            void *   AllocArray( Int32 ArraySize) { return  AllocArray_(DataTypeHandle,ArraySize); }





            /// <summary> Allocates a scalar of the given type and copies data from the area indicated by the void *  </summary>
            /// <param name="ValuePtr"> void *   to the data that is copied to the scalar being defined</param>
            /// <returns> void *   of the scalar that was allocated</returns>
            void *   AllocByPtr( void *   ValuePtr) { return  AllocByPtr_(DataTypeHandle,ValuePtr); }





            /// <summary> Allocates a scalar of the given data type</summary>
            /// <param name="Value"> Expression who&#39;s evaluated value will be copied to the allocated scalar</param>
            /// <returns> void *   of the scalar that was allocated</returns>
            void *   AllocScalar() { return  AllocScalar_(DataTypeHandle,EMPTYSTR); }

            void *   AllocScalar( CONSTSTRING Value) { return  AllocScalar_(DataTypeHandle,Value); }





            /// <summary> Returns the size (in bytes) of one unit of data for this data type</summary>
            Int32 ByteSize() { return  ByteSize_(DataTypeHandle); }





            /// <summary> Returns the index of the data type</summary>
            /// <remarks><para>The return value is a BuiltInTypesEnum enumerator value that represents the data type of the
            /// current object.  For instance, if the data type is a string, the return value would be Type_String.
            /// </para></remarks>
            BuiltInTypesEnum Index() { return  Index_(DataTypeHandle); }





            /// <summary> Returns the uCalc.Item handle of the data type</summary>
            /// <remarks><para>Each data type object can also be represented as a uCalc object by using this return value.</para></remarks>
            Item  Item() { uCalc::Item TempObj(Item_Overload_DataType_(DataTypeHandle)); return TempObj; }





            /// <summary> Returns the name of the data type</summary>
            STRING  Name() { RETURNSTRING(Name_Overload_DataType_(DataTypeHandle)); }





            /// <summary> Releases an array that was allocated with AllocArray</summary>
            /// <param name="ValuePtr"> void *   of uCalc array to be released from memory</param>
            /// <remarks><para>This should only be used for arrays that were explicitly allocated with uCalc&#39;s AllocArray.</para></remarks>
            void ReleaseArray( void *   ValuePtr) {  ReleaseArray_(DataTypeHandle,ValuePtr); }





            /// <summary> Releases a scalar that was allocated with AllocScalar</summary>
            /// <param name="ValuePtr"> void *   of uCalc scalar to be released from memory</param>
            /// <remarks><para>This should only be used for scalars that were explicitly allocated with uCalc&#39;s AllocScalar.</para></remarks>
            void ReleaseScalar( void *   ValuePtr) {  ReleaseScalar_(DataTypeHandle,ValuePtr); }





            /// <summary> Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary>
            /// <param name="ValuePtr"> void *   of the uCalc array to be reset</param>
            /// <remarks><para>Resets all the elements of the array to the default value for the given data type.  This should only
            /// be used for arrays that were explicitly allocated with uCalc&#39;s AllocArray.</para></remarks>
            void ResetArray( void *   ValuePtr) {  ResetArray_(DataTypeHandle,ValuePtr); }





            /// <summary> Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary>
            /// <param name="ValuePtr"> void *   of the uCalc scalar to be reset</param>
            void ResetScalar( void *   ValuePtr) {  ResetScalar_(DataTypeHandle,ValuePtr); }





            /// <summary> Sets the value of an array at a given memory location that was allocated by AllocArray</summary>
            /// <param name="Destination"> void *   of array (base address) who&#39;s element value is to be set</param>
            /// <param name="Source"> void *   to a value of the same data type that will be copied to the element</param>
            /// <param name="Index"> Index of the array element who&#39;s value needs to be set</param>
            void SetArray( void *   Destination ,  void *   Source ,  Int32 Index) {  SetArray_(DataTypeHandle,Destination , Source , Index); }





            /// <summary> Sets the value of a scalar at a given memory location that was allocated by AllocScalar</summary>
            /// <param name="Destination"> void *   of scalar who&#39;se value is to be set</param>
            /// <param name="Source"> void *   to a value of the same data type that will be copied to the element</param>
            void SetScalar( void *   Destination ,  void *   Source) {  SetScalar_(DataTypeHandle,Destination , Source); }





            /// <summary> swaps the pointers of two variables that were allocated with AllocScalar or AllocArray</summary>
            /// <param name="ValuePtrA"> void *   of variable 1</param>
            /// <param name="ValuePtrB"> void *   of variable 2</param>
            void Swap( void *   ValuePtrA ,  void *   ValuePtrB) {  Swap_(DataTypeHandle,ValuePtrA , ValuePtrB); }





            /// <summary> Retuns a string representing the value at the given address formatted for the given data type</summary>
            /// <param name="ValuePtr"> void *   of the data who&#39;s value you want to display</param>
            /// <remarks><para>If you have a void *   to data of the given type, you can display this data as a string with this function.  Special formatting
            /// associated with a data type is used, if such formatting was defined.  For instance, if you&#39;re dealing with a bool, then
            /// you might get: True, Yes, On, etc, depending on how the formatting was configured with SetOutput.</para></remarks> 
            STRING  ToString( void *   ValuePtr) { RETURNSTRING(ToString_Overload_DataType_ByPtr_(DataTypeHandle,ValuePtr)); }





            /// <summary> Retuns a formatted string of the given data type representing the value that was passed as text</summary>
            /// <param name="Value"> String representing data of any type that is to be represented as data of the given type</param>
            /// <remarks><para>If you pass a string, it will be converted to the given data type, and return using the formatting
            /// for that data type.</para></remarks>  
            STRING  ToString( CONSTSTRING Value ) { RETURNSTRING(ToString_Overload_DataType_ByStr_(DataTypeHandle,Value)); }






            /// <summary> Returns the handle of the data type object</summary>
            /// <remarks><para>Generally for internal use</para></remarks>
            void *    Handle() { return  Handle_Overload_DataType_(DataTypeHandle); }



            DataType() { DataTypeHandle = 0; }
            DataType( uCalcPtr PassedDataTypeHandle) { DataTypeHandle = PassedDataTypeHandle; }
            DataType(DataType % DataTypeHandleInstance) { DataTypeHandle = DataTypeHandleInstance.DataTypeHandle; }
            DataType % operator= (DataType % DataTypeHandleInstance) { DataTypeHandle = DataTypeHandleInstance.DataTypeHandle; return *this; }  
        };



        /// <summary> Adds an error handler</summary>
        /// <param name="ErrorHandlerAddress"> Address of the callback function containing the error handler code</param>
        /// <param name="Position"> Position in which to insert the error handler</param>
        /// <returns> The error handler can later be removed using this return object</returns>
        /// <remarks><para>Use this to set a a callback function that is called whenever there is an error.  Your callback can
        /// do things like alert the user of the error, correct the error and resume, keep a log of errors, etc.
        /// </para><para>
        /// By default, if there are multiple error handlers, they are called starting with the most recently
        /// defined one first, ending with the first one to be defined last.  In order to define a handler in a different
        /// order, specify the Position value of where it should be inserted; 0 so that it is called first, -1 so it
        /// is called last, or a positive integer indicated where it should be inserted.  If the Postion value is out
        /// of bounds, it is adjusted into the maximum acceptable value.
        /// </para><para>
        /// If you set ErrorResponse to ErrorHandlerResponseEnum.ErrorResume, subsequent error handler callbacks will
        /// be bypassed and uCalc will resume operations as though no error had taken place.  Note that the condition
        /// that caused the error must be corrected before resuming, otherwise uCalc may get stuck in an infinite loop.
        /// </para><para>
        /// Some functions, such as ErrorExpression, ErrorLocation, and ErrorSymbol produce a meaningful result only
        /// during the parsing stage.  They do not cause other problems beyond meaningless data during the evaluation stage.
        /// </para><para>
        /// For Floating Point Environment errors, such as: Err_Float_Division_By_Zero, Err_Float_Inexact_Result,
        /// Err_Float_Invalid_Operation, Err_Float_Overflow, and Err_Float_Underflow, the uCalc error handler callbacks
        /// are called only if the corresponding environment bits are not masked.
        /// </para><para>
        /// </para></remarks> 
        Item AddErrorHandler( UCCALLBACK ErrorHandlerAddress) { uCalc::Item TempObj(AddErrorHandler_(uCalcHandle,ErrorHandlerAddress ,0)); return TempObj; }

        Item AddErrorHandler( UCCALLBACK ErrorHandlerAddress, Int32 Position) { uCalc::Item TempObj(AddErrorHandler_(uCalcHandle,ErrorHandlerAddress ,Position)); return TempObj; }





        /// <summary> Creates a new uCalc instance</summary>
        /// <remarks><para>This is not meant to be called directly.</para></remarks>
        uCalcPtr CreateuCalc() { return  CreateuCalc_(uCalcHandle); }





        /// <summary> Returns a data type object</summary>
        /// <param name="Index"> Index associated with the data type</param>
        /// <remarks><para>Each instance of uCalc has its own separate handle &amp; data type object for any given data type.  For
        /// instance, ItemOf(&quot;Int64&quot;) in one instance of uCalc does not share the same handle as ItemOf(&quot;Int64&quot;) in
        /// another instance.  This allows the same data type to have a different name and different formatting in
        /// different instances.  However, they share the same data type index across instances.  Given the index
        /// of a data type, you can directly obtain the proper data type object for a particular uCalc instance,
        /// faster than obtaining it by name as text with ItemOf.</para></remarks>
        DataType DataTypeOfIndex( BuiltInTypesEnum Index) { uCalc::DataType TempObj(DataTypeOfIndex_(uCalcHandle,Index)); return TempObj; }





        /// <summary> Defines a uCalc.Item object (a constunction, function, operator, variable, etc)</summary>
        /// <param name="Expression"> Text containing item to be defined.  Expression must start with a keyword such as Function, Operator, Variable, Token, Syntax, etc., followed by a colon, before the expression</param>
        /// <param name="Address"> Address a function or variable is attached to</param>
        /// <param name="ExistingItem"> Address of existing item this definition is associated with</param>
        /// <remarks><para>Functions, operators, variables, constants, tokens, syntax constructs, etc., can be defined with
        /// this method.  Although there are specialized methods that do the same things, such as DefineFunction,
        /// DefineOperator, DefineVariable, DefineConstant, and DefineSyntax, this method may in same cases offer
        /// more flexibility.
        /// </para><para>
        /// The expression must be preceded by a keyword and colon, such as &quot;Variable:&quot;.  For a full list of keywords,
        /// see Definition Keywords.
        /// </para><para>
        /// Sections are separated by ~~ (two tilda signs).
        /// </para></remarks> 
        Item Define( CONSTSTRING Expression) { uCalc::Item TempObj(Define_(uCalcHandle,Expression ,0,0)); return TempObj; }

        Item Define( CONSTSTRING Expression, void *  Address) { uCalc::Item TempObj(Define_(uCalcHandle,Expression ,Address,0)); return TempObj; }

        Item Define( CONSTSTRING Expression, void *  Address, void *  ExistingItem) { uCalc::Item TempObj(Define_(uCalcHandle,Expression ,Address,ExistingItem)); return TempObj; }






        /// <summary> Defines a constant</summary>
        /// <param name="Definition"> Text containing constant to be defined</param>
        /// <param name="VariableAddress"> Address the constant is attached to</param>
        /// <remarks><para>
        /// A constant definition is similar to that of a variable defined with DefineVariable, except that the end-user is not allowed to modify the value of a constant.
        /// You are, however, allowed to perform lower-level operations such as renaming a constant, or deleting it and re-defining a new constant or variable with the same
        /// name but different value.  If you attach the constant to the address of a variable in your host program, the host program may change the value.  However, when
        /// the end-user attempts to assign a value to the constant, an error is raised.
        /// </para></remarks> 
        Item DefineConstant( CONSTSTRING Definition) { uCalc::Item TempObj(DefineConstant_(uCalcHandle,Definition ,0)); return TempObj; }

        Item DefineConstant( CONSTSTRING Definition, void *  VariableAddress) { uCalc::Item TempObj(DefineConstant_(uCalcHandle,Definition ,VariableAddress)); return TempObj; }






        /// <summary> Defines a data type</summary>
        /// <param name="TypeSize"> Number of bytes 1 scalar value of this type takes</param>
        /// <param name="AllocArrayValue"> Address of a function that will allocate an array of this type</param>
        /// <param name="AllocScalar"> Address of a function that will allocate a scalar of this type.  If a string is passed, this function must convert it to this type and store the value, otherwise the function must store a default value for this type.</param>
        /// <param name="AllocScalarByPtr"> Address of a function that takes a void *   to a value of this type, allocates a scalar and stores the copied value in the allocated space.</param>
        /// <param name="DeleteArray"> Address of a function that frees a memory block that was allocated with AllocValueArray</param>
        /// <param name="DeleteScalar"> Address of a function that frees a memory block that was allocated by AllocScalar or AllocScalarByPtr.</param>
        /// <param name="ResetScalar"> Address of a function that resets an allocated scalar back to the default value of this type.</param>
        /// <param name="ResetArray"> Address of a function that resets array elements back to the default value of this type.</param>
        /// <param name="SetValueArray"> Address of a function that stores an array element value.</param>
        /// <param name="SetValueScalar"> Address of a function that stores a scalar value.</param>
        /// <param name="SwapValues"> Address of a function that swaps two values of this type.</param>
        /// <param name="ToString"> Address of a function that converts this data type to a string.</param>
        /// <remarks><para>
        /// This allows you to define a data type that functions at the same level as the intrinsic types that come with uCalc.
        /// </para></remarks> 
        Item DefineDataType( Int32 TypeSize ,  void *   AllocArrayValue ,  void *   AllocScalar ,  void *   AllocScalarByPtr ,  void *   DeleteArray ,  void *   DeleteScalar ,  void *   ResetScalar ,  void *   ResetArray ,  void *   SetValueArray ,  void *   SetValueScalar ,  void *   SwapValues ,  void *   ToString) { uCalc::Item TempObj(DefineDataType_(uCalcHandle,TypeSize , AllocArrayValue , AllocScalar , AllocScalarByPtr , DeleteArray , DeleteScalar , ResetScalar , ResetArray , SetValueArray , SetValueScalar , SwapValues , ToString)); return TempObj; }





        /// <summary> Defines a function</summary>
        /// <param name="Definition"> Text containing function to be defined</param>
        /// <param name="FunctionAddress"> Address of callback function this function is attached to</param>
        /// <remarks><para>
        /// A function definition can either be self-contained, or it may be attached to the address of a callback function in your
        /// source code.
        /// </para></remarks> 
        Item DefineFunction( CONSTSTRING Definition) { uCalc::Item TempObj(DefineFunction_(uCalcHandle,Definition ,nullptr)); return TempObj; }

        Item DefineFunction( CONSTSTRING Definition, UCCALLBACK FunctionAddress) { uCalc::Item TempObj(DefineFunction_(uCalcHandle,Definition ,FunctionAddress)); return TempObj; }






        /// <summary> Defines an operator</summary>
        /// <param name="Definition"> Text containing operator to be defined</param>
        /// <param name="iPrecedence"> Precedence level of the operator.</param>
        /// <param name="Grouping"> Operator associativity; arithmetic operators are typically grouped left-to-right; special operators, such as the equal sign, =, assignment operator are right-to-left</param>
        /// <param name="FunctionAddress"> Address of callback function this operator is attached to</param>
        /// <remarks><para>
        /// Many details for defining an operator are similar to those of defining a function.  In additionn, all operators must be
        /// defined with a precedence level.  You may use the Precedence function to obtain precedence levels of existing operators,
        /// and select a precedence level in comparison to those operators.  By default operators are grouped left to right.  But you
        /// may also set it to right to left.  This means that when adjacent operators have the same precedence level, the operands on
        /// the right will be evaluated first.
        /// </para><para>
        /// More later
        /// </para></remarks> 
        Item DefineOperator( CONSTSTRING Definition  ,  Int32 iPrecedence) { uCalc::Item TempObj(DefineOperator_(uCalcHandle,Definition , iPrecedence ,GroupLeftToRight,nullptr)); return TempObj; }

        Item DefineOperator( CONSTSTRING Definition  ,  Int32 iPrecedence, GroupingEnum Grouping) { uCalc::Item TempObj(DefineOperator_(uCalcHandle,Definition , iPrecedence ,Grouping,nullptr)); return TempObj; }

        Item DefineOperator( CONSTSTRING Definition  ,  Int32 iPrecedence, GroupingEnum Grouping, UCCALLBACK FunctionAddress) { uCalc::Item TempObj(DefineOperator_(uCalcHandle,Definition , iPrecedence ,Grouping,FunctionAddress)); return TempObj; }






        /// <summary> Defines a syntax construct</summary>
        /// <param name="Definition"> Text containing syntax construct</param>
        Item DefineSyntax( CONSTSTRING Definition ) { uCalc::Item TempObj(DefineSyntax_(uCalcHandle,Definition)); return TempObj; }






        /// <summary> Defines a transform</summary>
        /// <param name="FromExpr"> Pattern representing the text that must be transformed</param>
        /// <param name="ToExpr"> The text that the matching test is transformed to</param>
        /// <param name="ItemHandle"> Handle</param> 
        Item DefineTransform( CONSTSTRING FromExpr  ,  CONSTSTRING ToExpr) { uCalc::Item TempObj(DefineTransform_(uCalcHandle,FromExpr , ToExpr ,0)); return TempObj; }

        Item DefineTransform( CONSTSTRING FromExpr  ,  CONSTSTRING ToExpr, void *  ItemHandle) { uCalc::Item TempObj(DefineTransform_(uCalcHandle,FromExpr , ToExpr ,ItemHandle)); return TempObj; }






        /// <summary> Defines a variable</summary>
        /// <param name="Definition"> Text containing variable to be defined</param>
        /// <param name="VariableAddress"> Address of variable in host program this uCalc variable is attached to</param>
        /// <remarks><para>A variable consists of ...</para></remarks> 
        Item DefineVariable( CONSTSTRING Definition) { uCalc::Item TempObj(DefineVariable_(uCalcHandle,Definition ,0)); return TempObj; }

        Item DefineVariable( CONSTSTRING Definition, void *  VariableAddress) { uCalc::Item TempObj(DefineVariable_(uCalcHandle,Definition ,VariableAddress)); return TempObj; }






        /// <summary> Returns the expression associated with an error that was raised</summary>
        STRING ErrorExpression() { RETURNSTRING(ErrorExpression_(uCalcHandle)); }





        /// <summary> Returns the location of an error that was raised</summary>
        Int32 ErrorLocation() { return  ErrorLocation_(uCalcHandle); }





        /// <summary> Returns a message associated with the error that was raised</summary>
        /// <param name="Index"> Index of error message to return</param>
        /// <remarks><para>If an error was raised, this returns a message describing the recent error.  If the Index argument is specified,
        /// then the error message associated with this Index is returned.  A value must be chosen from ErrorNumberEnum.
        /// </para><para>
        /// Note: Errors are cleared whenever a new expression is parsed or a new item is defined.
        /// </para></remarks> 
        STRING ErrorMessage() { RETURNSTRING(ErrorMessage_(uCalcHandle,Err_None)); }

        STRING ErrorMessage( ErrorNumberEnum Index) { RETURNSTRING(ErrorMessage_(uCalcHandle,Index)); }





        /// <summary> Returns the error number of the error that was raised</summary>
        /// <remarks><para>
        /// Returns the error number of the most recently raised error.
        /// </para><para>
        /// Note: Errors are cleared whenever a new expression is parsed or a new item is defined.
        /// </para></remarks>
        ErrorNumberEnum ErrorNumber() { return  ErrorNumber_(uCalcHandle); }





        /// <summary> Sets the error response for an error handler</summary>
        /// <param name="Response"> Indicates what should happen now that an error is raised</param>
        /// <remarks><para>When an error is raised, you have the option of aborting the rest of the option (which is the default option),
        /// or you may correct the problem, and then resume the operation.  Or you may perform an action, and than raise the
        /// error again for a subsequent error handle to take care of.
        /// </para><para>
        /// This method can only be called from within a callback of an error handler routine definined with AddErrorHandler.
        /// The options to use can be found at ErrorHandlerResponseEnum.
        /// </para></remarks> 
        void ErrorResponse( ErrorHandlerResponseEnum Response) {  ErrorResponse_(uCalcHandle,Response); }





        /// <summary> Returns the symbol associated with the error that is raised</summary>
        STRING ErrorSymbol() { RETURNSTRING(ErrorSymbol_(uCalcHandle)); }





        /// <summary> Sets a new expression after an error is raised</summary>
        /// <param name="Expression"> Error expression</param>
        void ErrorSetExpression( CONSTSTRING Expression ) {  ErrorSetExpression_(uCalcHandle,Expression); }






        /// <summary> Parses and evaluates an expression in one step</summary>
        /// <param name="Expression"> Expression to be evaluated</param>
        /// <remarks><para>This takes a math expression and returns a numeric result.</para></remarks> 
        double Eval( CONSTSTRING Expression ) { return  Eval_(uCalcHandle,Expression); }






        /// <summary> Parses and evaluates an expression of any data type in one step, returning the result as a string</summary>
        /// <param name="Expression"> Expression to be evaluated</param>
        /// <remarks><para>This is similar to Eval, however, it can return non-numeric results, such as strings.</para></remarks> 
        STRING EvalStr( CONSTSTRING Expression ) { RETURNSTRING(EvalStr_(uCalcHandle,Expression)); }






        /// <summary> Expands an expression</summary>
        /// <param name="Expression"> Expression to be expanded</param>
        STRING Expand( CONSTSTRING Expression ) { RETURNSTRING(Expand_(uCalcHandle,Expression)); }






        /// <summary> Returns each step leading to an expression expansion</summary>
        /// <param name="Expression"> Expression to be expanded</param>
        /// <param name="Separator"> Separator used between steps</param>
        STRING ExpandSteps( CONSTSTRING Expression) { RETURNSTRING(ExpandSteps_(uCalcHandle,Expression ,EMPTYSTR)); }

        STRING ExpandSteps( CONSTSTRING Expression, CONSTSTRING Separator) { RETURNSTRING(ExpandSteps_(uCalcHandle,Expression ,Separator)); }






        /// <summary> Clears floating point exception(s) that were raised</summary>
        /// <param name="Exceptions"> Exceptions</param>
        Int32 FPEnvClearException() { return  FPEnvClearException_(uCalcHandle,0); }

        Int32 FPEnvClearException( Int32 Exceptions) { return  FPEnvClearException_(uCalcHandle,Exceptions); }





        /// <summary> Returns the floating point environment error mask</summary>
        Int32 FPEnvGetExceptionMask() { return  FPEnvGetExceptionMask_(uCalcHandle); }





        /// <summary> Raises a floating point exception in a uCalc.Callback function</summary>
        /// <param name="Exception"> Bitmask value consisting of one or more exception flags to raise</param>
        /// <returns> Returns a 0 if all the specified execptions were raised successfully, otherwise a non-zero value is returned.</returns>
        /// <remarks><para>This attempts to raise the exception(s) specfied by the Exception parameter.  This is based on the C++
        /// feraiseexcept() function.  If you are using C++, you may use the pre-defined macro values of FE_DIVBYZERO,
        /// FE_INEXACT, FE_INVALID, FE_OVERFLOW, FE_UNDERFLOW, FE_ALL_EXCEPT.  If you are using another compiler, you may
        /// use the matching members of the FPEnvExceptionEnum enumerator.
        /// </para></remarks>
        Int32 FPEnvRaiseException() { return  FPEnvRaiseException_(uCalcHandle,Invalid); }

        Int32 FPEnvRaiseException( FPEnvExceptionEnum Exception) { return  FPEnvRaiseException_(uCalcHandle,Exception); }





        /// <summary> Gets the current floating point environment rounding option</summary>
        FPEnvRoundingEnum FPEnvGetRounding() { return  FPEnvGetRounding_(uCalcHandle); }





        /// <summary> Sets the current floating point environment mask</summary>
        void FPEnvSetExceptionMask( Int32 Exceptions) {  FPEnvSetExceptionMask_(uCalcHandle,Exceptions); }





        /// <summary> Unmasks the current floating point environment mask</summary>
        void FPEnvSetExceptionUnMask( Int32 Exceptions) {  FPEnvSetExceptionUnMask_(uCalcHandle,Exceptions); }





        /// <summary> Sets the floating point environment rounding option</summary>
        Int32 FPEnvSetRounding( FPEnvRoundingEnum Direction) { return  FPEnvSetRounding_(uCalcHandle,Direction); }





        /// <summary> Toggles the floating point environment mask</summary>
        void FPEnvToggleExceptionMask( FPEnvExceptionEnum Exception) {  FPEnvToggleExceptionMask_(uCalcHandle,Exception); }





        /// <summary> Returns the handle of a uCalc.Item given a name</summary>
        uCalcPtr HandleOf( CONSTSTRING ItemName) { return  HandleOf_(uCalcHandle,ItemName, 0, false); }

        uCalcPtr HandleOf( CONSTSTRING ItemName, Int64 Properties) { return  HandleOf_(uCalcHandle,ItemName, Properties, false); }

        uCalcPtr HandleOf( CONSTSTRING ItemName, Int64 Properties, bool SkipExpansion) { return  HandleOf_(uCalcHandle,ItemName, Properties, SkipExpansion); }






        /// <summary> Returns the uCalc.Item object associated with the given name</summary>
        /// <param name="ItemName"> Name of uCalc.Item that you want to get</param>
        /// <param name="Properties"> Properties to help pinpoint the right Item, in case several items share the same name</param>
        /// <param name="SkipExpansion"> Some items are define as a syntax construct alias of another item.  By default the symbol is expanded first.  However, use this option if the alias is the object you&#39;re actually after.</param>
        /// <remarks><para>
        /// If there is no match, an item whose IsProperty ItemIsEnum.NotFound value is true is returned.
        /// </para><para>
        /// In case multiple items share the same name, you can narrow it down using properties.  For instance, the + operator has a
        /// binary and unary versions.  You can specify Prefix if you wanted the unary one.
        /// </para><para>
        /// Note: A ItemIsEnum member should not be passed directly as the second argument.  Instead Properties, with one or more
        /// ItemIsEnum members should be passed.
        /// </para></remarks>
        Item  ItemOf( CONSTSTRING ItemName) { uCalc::Item TempObj(ItemOf_Overload_uCalc_ByName_(uCalcHandle,ItemName ,0,false)); return TempObj; }

        Item  ItemOf( CONSTSTRING ItemName, Int64 Properties) { uCalc::Item TempObj(ItemOf_Overload_uCalc_ByName_(uCalcHandle,ItemName ,Properties,false)); return TempObj; }

        Item  ItemOf( CONSTSTRING ItemName, Int64 Properties, bool SkipExpansion) { uCalc::Item TempObj(ItemOf_Overload_uCalc_ByName_(uCalcHandle,ItemName ,Properties,SkipExpansion)); return TempObj; }






        /// <summary> Returns the nth uCalc.Item object that matches a property</summary>
        /// <param name="Property_"> Property of the uCalc.Item object you&#39;re after</param>
        /// <param name="nth"> The nth object</param>
        /// <param name="DataTypeName"> The data type of the object</param>
        /// <remarks><para>
        /// This can be used to cycle through defined items.
        /// If there is no match, an item whose IsProperty ItemIsEnum.NotFound value is true is returned.
        /// </para></remarks>
        Item  ItemOf( ItemIsEnum Property_ ,  Int32 nth) { uCalc::Item TempObj(ItemOf_Overload_uCalc_ByProperty_(uCalcHandle,Property_ , nth ,EMPTYSTR)); return TempObj; }

        Item  ItemOf( ItemIsEnum Property_ ,  Int32 nth, CONSTSTRING DataTypeName) { uCalc::Item TempObj(ItemOf_Overload_uCalc_ByProperty_(uCalcHandle,Property_ , nth ,DataTypeName)); return TempObj; }








        /// <summary> Sets the license code</summary>
        Int32 License( LicenseTypeEnum LicenseType) { return  License_(uCalcHandle,LicenseType, 0, 0, 0, 0, 0); }

        Int32 License( LicenseTypeEnum LicenseType, Int32 L1) { return  License_(uCalcHandle,LicenseType, L1, 0, 0, 0, 0); }

        Int32 License( LicenseTypeEnum LicenseType, Int32 L1, Int32 L2) { return  License_(uCalcHandle,LicenseType, L1, L2, 0, 0, 0); }

        Int32 License( LicenseTypeEnum LicenseType, Int32 L1, Int32 L2, Int32 L3) { return  License_(uCalcHandle,LicenseType, L1, L2, L3, 0, 0); }

        Int32 License( LicenseTypeEnum LicenseType, Int32 L1, Int32 L2, Int32 L3, Int32 L4) { return  License_(uCalcHandle,LicenseType, L1, L2, L3, L4, 0); }

        Int32 License( LicenseTypeEnum LicenseType, Int32 L1, Int32 L2, Int32 L3, Int32 L4, Int32 L5) { return  License_(uCalcHandle,LicenseType, L1, L2, L3, L4, L5); }





        /// <summary> ???</summary>
        Item NamespaceChild() { uCalc::Item TempObj(NamespaceChild_(uCalcHandle,0)); return TempObj; }

        Item NamespaceChild( Int32 nth) { uCalc::Item TempObj(NamespaceChild_(uCalcHandle,nth)); return TempObj; }





        /// <summary> ???</summary>
        Int32 NamespaceChildCount() { return  NamespaceChildCount_(uCalcHandle); }





        /// <summary> ???</summary>
        Item NamespaceParent() { uCalc::Item TempObj(NamespaceParent_(uCalcHandle,0)); return TempObj; }

        Item NamespaceParent( Int32 Level) { uCalc::Item TempObj(NamespaceParent_(uCalcHandle,Level)); return TempObj; }





        /// <summary> ???</summary>
        Item NamespaceSymbol( Int32 nth) { uCalc::Item TempObj(NamespaceSymbol_(uCalcHandle,nth)); return TempObj; }





        /// <summary> ???</summary>
        Int32 NamespaceSymbolCount() { return  NamespaceSymbolCount_(uCalcHandle); }





        /// <summary> Parses an expression that will later be evaluated with Evaluate or EvaluateStr</summary>
        /// <param name="Expression"> Expression to be parsed</param>
        /// <param name="DataType"> Data type of return value</param>
        ParsedExpr Parse( CONSTSTRING Expression) { uCalc::ParsedExpr TempObj(Parse_(uCalcHandle,Expression ,EMPTYSTR)); return TempObj; }

        ParsedExpr Parse( CONSTSTRING Expression, CONSTSTRING DataType) { uCalc::ParsedExpr TempObj(Parse_(uCalcHandle,Expression ,DataType)); return TempObj; }






        /// <summary> Returns the precedence level of an operator or the level between two operators</summary>
        Int32  Precedence( CONSTSTRING Operator1) { return  Precedence_Overload_uCalc_(uCalcHandle,Operator1, EMPTYSTR); }

        Int32  Precedence( CONSTSTRING Operator1, CONSTSTRING Operator2) { return  Precedence_Overload_uCalc_(uCalcHandle,Operator1, Operator2); }






        /// <summary> Combines properties into a numeric value that can be passed to ItemOf</summary>
        /// <remarks><para>
        /// ItemIsEnum properties are numbered sequentially (0, 1, 2, 3, 4, ...).  However the Properties argument
        /// of the ItemOf (ByName) function requires properties in the 2^x sequence (1, 2, 4, 8, 16, ...).  So you
        /// must pass one or more ItemIsEnum to Properties before passing it to ItemOf (ByName).
        /// </para></remarks>
        Int64 Properties( ItemIsEnum p1) { return  Properties_(uCalcHandle,p1, AnyItem, AnyItem, AnyItem); }

        Int64 Properties( ItemIsEnum p1, ItemIsEnum p2) { return  Properties_(uCalcHandle,p1, p2, AnyItem, AnyItem); }

        Int64 Properties( ItemIsEnum p1, ItemIsEnum p2, ItemIsEnum p3) { return  Properties_(uCalcHandle,p1, p2, p3, AnyItem); }

        Int64 Properties( ItemIsEnum p1, ItemIsEnum p2, ItemIsEnum p3, ItemIsEnum p4) { return  Properties_(uCalcHandle,p1, p2, p3, p4); }





        /// <summary> (Use Release instead)</summary>
        /// <remarks><para>Use Release</para></remarks> 
        void ReleaseuCalc() {  ReleaseuCalc_(uCalcHandle); }





        /// <summary> Removes any output decoration</summary>
        void ResetOutput() {  ResetOutput_(uCalcHandle); }





        /// <summary> Returns default settings</summary>
        /// <remarks><para>The Item returned contains the default settings for the current uCalc instance.  To define new settings,
        /// use Define(&quot;Default ~~ ...&quot;).</para></remarks> 
        Item Default() { uCalc::Item TempObj(Default_(uCalcHandle)); return TempObj; }





        /// <summary> Allows a callback to decorate the output of EvalStr</summary>
        Item SetOutput( UCCALLBACK FunctionAddress,  CONSTSTRING Definition ) { uCalc::Item TempObj(SetOutput_(uCalcHandle,FunctionAddress, Definition)); return TempObj; }






        /// <summary> Returns the value located at the specified void *   address</summary>  
        STRING ValueAt( void *   ValuePtr,  CONSTSTRING DataType ) { RETURNSTRING(ValueAt_(uCalcHandle,ValuePtr, DataType)); }






        /// <summary> Returns the handle of the uCalc object</summary>
        /// <remarks><para>Generally for internal use</para></remarks>
        void *    Handle() { return  Handle_Overload_uCalc_(uCalcHandle); }




        void Release() { ReleaseuCalc(); uCalcHandle = 0; }
        /// <summary> Releases a uCalc instance</summary>
        /// <remarks><para>When a uCalc instance is no longer needed, it can be released with this function.  This releases all items
        /// associated with the uCalc instance.
        /// </para><para>
        /// Note: The uCalc object named uc, which is instantiated at startup (except in C#) should not be explicitely released.
        /// In C#, if there is only 1 uCalc object left, do not explicitely release it.
        /// </para><para>
        /// Note: A uCalc object must explicitely be released with Release for it to be released.  A uCalc object is not
        /// automatically released from memory when the object goes out of scope.</para></remarks>


        void Reset() { ReleaseuCalc(); uCalcHandle = CreateuCalc(); }
        uCalc() {


            uCalcHandle = 0; uCalcHandle = CreateuCalc();
        }
        uCalc(uCalc % uCalcInstance) { uCalcHandle = uCalcInstance.uCalcHandle; }
        uCalc % operator= (uCalc % uCalcInstance) { uCalcHandle = uCalcInstance.uCalcHandle; return *this; }  
        uCalc( uCalcPtr InstHandle) { uCalcHandle = InstHandle; }
        uCalc( LicenseTypeEnum LicenseType) { License(LicenseType, 0, 0, 0, 0, 0); uCalcHandle = CreateuCalc(); }

        uCalc( LicenseTypeEnum LicenseType, Int32 L1) { License(LicenseType, L1, 0, 0, 0, 0); uCalcHandle = CreateuCalc(); }

        uCalc( LicenseTypeEnum LicenseType, Int32 L1, Int32 L2) { License(LicenseType, L1, L2, 0, 0, 0); uCalcHandle = CreateuCalc(); }

        uCalc( LicenseTypeEnum LicenseType, Int32 L1, Int32 L2, Int32 L3) { License(LicenseType, L1, L2, L3, 0, 0); uCalcHandle = CreateuCalc(); }

        uCalc( LicenseTypeEnum LicenseType, Int32 L1, Int32 L2, Int32 L3, Int32 L4) { License(LicenseType, L1, L2, L3, L4, 0); uCalcHandle = CreateuCalc(); }

        uCalc( LicenseTypeEnum LicenseType, Int32 L1, Int32 L2, Int32 L3, Int32 L4, Int32 L5) { License(LicenseType, L1, L2, L3, L4, L5); uCalcHandle = CreateuCalc(); }
    }; // uCalc







#if !defined SkipuCalcAutoDef

#endif

} // End of uCalcNameSpace
using namespace uCalcNameSpace;
